#lang racket

;;; LAScheme -- or, AspectScheme with Execution Levels
;;;  by Éric Tanter
;;; 
;;; AspectScheme was originally developed by Christopher Dutchyn.
;;; This is an extension of AspectScheme that supports execution levels.
;;; (see "Execution Levels for Aspect-Oriented Programming", by Tanter, AOSD 2010)
;;; main differences with AspectScheme:
;;; - execution of pointcuts and advices happens at a higher level
;;; - no pc/adv join points (not needed anymore, thanks to levels)
;;; - app/prim is not exposed anymore (not needed, thanks to levels)
;;; - exports level-shifting operators ^up and ^down
;;; - exports lambda-s to define level-capturing functions
;;; - changed the interface of the language for creating and deploying aspects
;;; - fixed minor details and bugs, and renamed some functions


;;; This module activates aspect weaving (it redefines function application and lambda definition).
;;; This means that a client module that imports this module is subject to aspect weaving.

(require "lascheme-api.rkt")

(provide (except-out (all-from-out "lascheme-api.rkt") lambda #%app app/weave lambda/aspects)
         (rename-out [app/weave #%app]
                     [lambda/aspects lambda]))