#lang racket

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Testing LAScheme
;;
;;

(require "lascheme.rkt")
(require "test.rkt")

(time
;;;;;; 
;; a level 0 aspect (writes 0 to stream)

;; a) no use of up/down in (*)

(let ((trace 
       (lambda (proceed) ;; proceed
         (lambda ctx ;; ctx
           (lambda (a) ;; val
             (write 'a) ;; (*) ;; writes which advice is executing
             (proceed a))))))
  
  ;; dynamically-scoped level 0
  (test/str
   (deploy-fluid (aspect (&& (call write) 
                             (lambda jp (write 'pc)'())) ;; (*)
                         trace)
                 (write 'hola))
   "pcahola"))
  ;; pc, adv, base


;;;;;; 
;; a level 1 aspect (writes 1 to stream)
;; a level 0 aspect (writes 0 to stream)

;; a) no use of up/down in (*)

(let ((trace (lambda (mark)
               (lambda (proceed) ;; proceed
                 (lambda ctx ;; ctx
                   (lambda (a) ;; val
                     (write mark) ;; (*) ;; writes which advice is executing
                     (proceed a)))))))

;; level 1
  (^up (deploy-fluid (aspect (call write) (trace 1)) 
    (^down             
    ;; dynamically-scoped level 0
    (test/str
     (deploy-fluid (aspect (&& (call write) 
                               (lambda jp (write 'pc)'())) ;; (*)
                           (trace 0))
                   (write 'hola))
     "1pc10hola")))))
  ;; adv 1, pc, adv 1, adv 0, base


;; a') same, but with top-level deployment

(let ((trace (lambda (mark)
               (lambda (proceed) ;; proceed
                 (lambda ctx ;; ctx
                   (lambda (a) ;; val
                     (write mark) ;; (*) ;; writes which advice is executing
                     (proceed a)))))))

  (let ((d1 (^up (deploy-top (aspect (call write) (trace 1)))))
        (d2 (deploy-top (aspect (&& (call write) 
                                    (lambda jp (write 'pc)'())) ;; (*)
                                (trace 0)))))
    (test/str
     (write 'hola)
     "1pc10hola")
    ;; clean up the global env
    (undeploy-top d1)
    (undeploy-top d2)))

;; b) move advice/pointcut writes to upper level

(let ((trace (lambda (mark)
               (lambda (proceed) ;; proceed
                 (lambda ctx ;; ctx
                   (lambda (a) ;; val
                     (^up (write mark)) ;; (*) ;; writes which advice is executing
                     (proceed a)))))))

  ;; level 1
  (^up 
   (deploy-fluid (aspect (call write) (trace 1))
     (^down

  ;; dynamically-scoped level 0
    (test/str
     (deploy-fluid (aspect (&& (call write) 
                               (lambda jp (^up (write 'pc))'())) ;; (*)
                           (trace 0))
                   (write 'hola))
     "pc0hola")))))
  ;; pc, adv 0, base [adv1 does not apply, since all is up'ed to l2]

;; Testing "invisibility" of aspect activities
(let ((jp-counter (let ((count 0))
                    (lambda ()
                      (lambda (proceed)
                        (if (eq? 'get-count proceed) ;; access the counter
                            count
                            (lambda ctx
                              (lambda args
                                (set! count (+ 1 count))  ;; just count jps
                                (apply proceed args)))))))))
  (let ((counter1 (jp-counter))
        (counter2 (jp-counter)))
    ;;jp counter on level 0
    (deploy-fluid  (aspect (lambda (jp+ jp jp-) '())    ;; true pc
                           counter1)
      (map (lambda (x) (+ x 1)) '(1 2 3 4 5)))
   
    ;; now same thing, with an aspect that stays at its level
    (deploy-fluid (aspect (lambda (jp+ jp jp-) (map zero? '(1 2 3)) ;; harmless activity
                            '())
                          (lambda (proceed)
                            (lambda ctx
                              (lambda args
                                (map zero? '(1 2 3)) ;; harmless activity
                                (apply proceed args)))))
      ;;jp counter on level 0
      (deploy-fluid  (aspect (lambda (jp+ jp jp-) '())    ;; true pc
                             counter2)
        (map (lambda (x) (+ x 1)) '(1 2 3 4 5))))
    
    ;; Introducing "pure" aspect should not affect number of join points observable at level 0
    (test (- (counter1 'get-count) (counter2 'get-count)) 0)))


;;;;;;;;;;;; testing level capturing function

;;; Case 1: lambda-s at the same level, so aspect sees it (ie. applies twice)
(let ((trace 
       (lambda (proceed) ;; proceed
         (lambda ctx ;; ctx
           (lambda (a) ;; val
             (write '-apply-)
             (proceed a))))))
  
  ;; dynamically-scoped level 0
  (test/str
   (deploy-fluid (aspect (call write) trace)
                 (write 'hola)
                 (let ((write-task (lambda-s (x) (write x))))
                   (write-task 'si)))
   "-apply-hola-apply-si"))

;;; Case 2: lambda-s at upper level, so aspect does not see it (ie. applies once)
(let ((trace 
       (lambda (proceed) ;; proceed
         (lambda ctx ;; ctx
           (lambda (a) ;; val
             (write '-apply-)
             (proceed a))))))
  
  ;; dynamically-scoped level 0
  (test/str
   (deploy-fluid (aspect (call write) trace)
                 (write 'hola)
                 (let ((write-task (^up (lambda-s (x) (write x)))))
                   (write-task 'si)))
   "-apply-holasi"))

;;; simple simulation of the schedule-task example from the AOSD submission:
(define task (lambda () "nothing"))
(define (schedule-task t) (set! task t))
(define (run-task) (task))

(define activity 
  (lambda (proceed)
    (lambda ctx
      (lambda args
        (schedule-task (lambda-s () (write 'logging))) ;; task captures level 1
        (write '-a-)
        (apply proceed args)))))

(test/str
 (deploy-fluid (aspect (call write) activity)
               (write 'hola)
               (run-task))  ;; aspect does not see the write here: matches only once
 "-a-holalogging")

)